@@ -16,10 +16,13 @@ def test_vendoring_cupy():
 
 
 def test_vendoring_torch():
+    pytest.importorskip("torch")
     from vendor_test import uses_torch
 
     uses_torch._test_torch()
 
+
 def test_vendoring_dask():
+    pytest.importorskip("dask")
     from vendor_test import uses_dask
     uses_dask._test_dask()
