@@ -638,12 +638,14 @@ def _linprog_simplex(c, c0, A, b, callback, postsolve_args,
         status = 2
         messages[status] = (
             "Phase 1 of the simplex method failed to find a feasible "
-            "solution. The pseudo-objective function evaluates to {0:.1e} "
-            "which exceeds the required tolerance of {1} for a solution to be "
+            "solution. The pseudo-objective function evaluates to "
+            f"{abs(T[-1, -1]):.1e} "
+            f"which exceeds the required tolerance of {tol} for a solution to be "
             "considered 'close enough' to zero to be a basic solution. "
-            "Consider increasing the tolerance to be greater than {0:.1e}. "
-            "If this tolerance is unacceptably  large the problem may be "
-            "infeasible.".format(abs(T[-1, -1]), tol)
+            "Consider increasing the tolerance to be greater than "
+            f"{abs(T[-1, -1]):.1e}. "
+            "If this tolerance is unacceptably large the problem may be "
+            "infeasible."
         )
 
     if status == 0:
