@@ -11,7 +11,7 @@ from scipy.optimize import differential_evolution, OptimizeResult
 from scipy.optimize._constraints import (Bounds, NonlinearConstraint,
                                          LinearConstraint)
 from scipy.optimize import rosen, minimize
-from scipy.sparse import csr_matrix
+from scipy.sparse import csr_array
 from scipy import stats
 
 import numpy as np
@@ -311,7 +311,7 @@ class TestDifferentialEvolutionSolver:
         callback.func = func
 
         bounds = [(0, 2), (0, 2)]
-        kwargs = dict(func=func, bounds=bounds, seed=838245, polish=False)
+        kwargs = dict(func=func, bounds=bounds, rng=838245, polish=False)
         res = differential_evolution(**kwargs, callback=callback)
         ref = differential_evolution(**kwargs, maxiter=maxiter)
 
@@ -376,7 +376,7 @@ class TestDifferentialEvolutionSolver:
         args = (1., 2., 3.)
 
         def quadratic(x, *args):
-            if type(args) != tuple:
+            if not isinstance(args, tuple):
                 raise ValueError('args should be a tuple')
             return args[0] + args[1] * x + args[2] * x**2.
 
@@ -493,18 +493,19 @@ class TestDifferentialEvolutionSolver:
         # test the quadratic function from differential_evolution function
         differential_evolution(self.quadratic,
                                [(-100, 100)],
-                               tol=0.02)
+                               tol=0.02,
+                               seed=1)
 
-    def test_seed_gives_repeatability(self):
+    def test_rng_gives_repeatability(self):
         result = differential_evolution(self.quadratic,
                                         [(-100, 100)],
                                         polish=False,
-                                        seed=1,
+                                        rng=1,
                                         tol=0.5)
         result2 = differential_evolution(self.quadratic,
                                         [(-100, 100)],
                                         polish=False,
-                                        seed=1,
+                                        rng=1,
                                         tol=0.5)
         assert_equal(result.x, result2.x)
         assert_equal(result.nfev, result2.nfev)
@@ -519,7 +520,7 @@ class TestDifferentialEvolutionSolver:
             differential_evolution(self.quadratic,
                                    [(-100, 100)],
                                    polish=False,
-                                   seed=rng,
+                                   rng=rng,
                                    tol=0.5,
                                    init=init)
 
@@ -636,7 +637,7 @@ class TestDifferentialEvolutionSolver:
         solver = DifferentialEvolutionSolver(rosen, self.bounds,
                                              init=population,
                                              strategy='best2bin',
-                                             atol=0.01, seed=1, popsize=5)
+                                             atol=0.01, rng=1, popsize=5)
 
         assert_equal(solver._nfev, 0)
         assert_(np.all(np.isinf(solver.population_energies)))
@@ -703,6 +704,7 @@ class TestDifferentialEvolutionSolver:
         )
         assert res.success
 
+    @pytest.mark.thread_unsafe
     def test_immediate_updating(self):
         # check setting of immediate updating, with default workers
         bounds = [(0., 2.), (0., 2.)]
@@ -719,10 +721,10 @@ class TestDifferentialEvolutionSolver:
             # is being overridden by the workers keyword
             with warns(UserWarning):
                 with DifferentialEvolutionSolver(rosen, bounds, workers=p.map) as s:
-                    pass
+                    solver.solve()
             assert s._updating == 'deferred'
 
-    @pytest.mark.fail_slow(5)
+    @pytest.mark.fail_slow(10)
     def test_parallel(self):
         # smoke test for parallelization with deferred updating
         bounds = [(0., 2.), (0., 2.)]
@@ -848,6 +850,7 @@ class TestDifferentialEvolutionSolver:
             assert_almost_equal(cv, np.array([[0.0, 0.0, 0.], [2.1, 4.2, 0]]))
             assert cv.shape == (2, 3)
 
+    @pytest.mark.thread_unsafe
     def test_constraint_solve(self):
         def constr_f(x):
             return np.array([x[0] + x[1]])
@@ -864,7 +867,8 @@ class TestDifferentialEvolutionSolver:
         assert constr_f(res.x) <= 1.9
         assert res.success
 
-    @pytest.mark.fail_slow(5)
+    @pytest.mark.fail_slow(10)
+    @pytest.mark.thread_unsafe
     def test_impossible_constraint(self):
         def constr_f(x):
             return np.array([x[0] + x[1]])
@@ -872,7 +876,7 @@ class TestDifferentialEvolutionSolver:
         nlc = NonlinearConstraint(constr_f, -np.inf, -1)
 
         solver = DifferentialEvolutionSolver(
-            rosen, [(0, 2), (0, 2)], constraints=(nlc,), popsize=1, seed=1, maxiter=100
+            rosen, [(0, 2), (0, 2)], constraints=(nlc,), popsize=1, rng=1, maxiter=100
         )
 
         # a UserWarning is issued because the 'trust-constr' polishing is
@@ -961,7 +965,7 @@ class TestDifferentialEvolutionSolver:
             violations.append(pc.violation(x))
         np.testing.assert_allclose(pc.violation(xs.T), np.array(violations).T)
 
-        pc = _ConstraintWrapper(LinearConstraint(csr_matrix(A), -np.inf, 0),
+        pc = _ConstraintWrapper(LinearConstraint(csr_array(A), -np.inf, 0),
                                 x0)
         assert (pc.violation(x0) > 0).any()
         assert (pc.violation([-10, 2, -10, 4]) == 0).all()
@@ -1021,7 +1025,7 @@ class TestDifferentialEvolutionSolver:
         xtrial = np.arange(4 * 5).reshape(4, 5)
         assert cw.violation(xtrial).shape == (2, 5)
 
-    @pytest.mark.fail_slow(10)
+    @pytest.mark.fail_slow(20)
     def test_L1(self):
         # Lampinen ([5]) test problem 1
 
@@ -1050,8 +1054,8 @@ class TestDifferentialEvolutionSolver:
 
         # using a lower popsize to speed the test up
         res = differential_evolution(
-            f, bounds, strategy='best1bin', seed=1234, constraints=(L,),
-            popsize=2, tol=0.05
+            f, bounds, strategy='best1bin', rng=12345, constraints=(L,),
+            popsize=5, tol=0.01
         )
 
         x_opt = (1, 1, 1, 1, 1, 1, 1, 1, 1, 3, 3, 3, 1)
@@ -1066,14 +1070,14 @@ class TestDifferentialEvolutionSolver:
         assert_(np.all(res.x <= np.array(bounds)[:, 1]))
 
         # now repeat the same solve, using the same overall constraints,
-        # but using a sparse matrix for the LinearConstraint instead of an
+        # but using a sparse array for the LinearConstraint instead of an
         # array
 
-        L = LinearConstraint(csr_matrix(A), -np.inf, b)
+        L = LinearConstraint(csr_array(A), -np.inf, b)
 
         # using a lower popsize to speed the test up
         res = differential_evolution(
-            f, bounds, strategy='best1bin', seed=1234, constraints=(L,),
+            f, bounds, strategy='best1bin', rng=1211134, constraints=(L,),
             popsize=2, tol=0.05
         )
 
@@ -1106,7 +1110,7 @@ class TestDifferentialEvolutionSolver:
         with suppress_warnings() as sup:
             sup.filter(UserWarning)
             res = differential_evolution(
-                f, bounds, strategy='best1bin', seed=1234,
+                f, bounds, strategy='best1bin', rng=1211134,
                 constraints=constraints, popsize=2, tol=0.05
             )
 
@@ -1116,7 +1120,7 @@ class TestDifferentialEvolutionSolver:
         assert_(np.all(res.x >= np.array(bounds)[:, 0]))
         assert_(np.all(res.x <= np.array(bounds)[:, 1]))
 
-    @pytest.mark.fail_slow(5)
+    @pytest.mark.fail_slow(10)
     def test_L2(self):
         # Lampinen ([5]) test problem 2
 
@@ -1142,7 +1146,7 @@ class TestDifferentialEvolutionSolver:
         with suppress_warnings() as sup:
             sup.filter(UserWarning)
             res = differential_evolution(f, bounds, strategy='best1bin',
-                                         seed=1234, constraints=constraints)
+                                         rng=1234, constraints=constraints)
 
         f_opt = 680.6300599487869
         x_opt = (2.330499, 1.951372, -0.4775414, 4.365726,
@@ -1156,7 +1160,7 @@ class TestDifferentialEvolutionSolver:
         assert_(np.all(res.x >= np.array(bounds)[:, 0]))
         assert_(np.all(res.x <= np.array(bounds)[:, 1]))
 
-    @pytest.mark.fail_slow(5)
+    @pytest.mark.fail_slow(10)
     def test_L3(self):
         # Lampinen ([5]) test problem 3
 
@@ -1191,7 +1195,7 @@ class TestDifferentialEvolutionSolver:
 
         with suppress_warnings() as sup:
             sup.filter(UserWarning)
-            res = differential_evolution(f, bounds, seed=1234,
+            res = differential_evolution(f, bounds, rng=1234,
                                          constraints=constraints, popsize=3)
 
         x_opt = (2.171996, 2.363683, 8.773926, 5.095984, 0.9906548,
@@ -1207,7 +1211,7 @@ class TestDifferentialEvolutionSolver:
         assert_(np.all(res.x >= np.array(bounds)[:, 0]))
         assert_(np.all(res.x <= np.array(bounds)[:, 1]))
 
-    @pytest.mark.fail_slow(5)
+    @pytest.mark.fail_slow(10)
     def test_L4(self):
         # Lampinen ([5]) test problem 4
         def f(x):
@@ -1235,7 +1239,7 @@ class TestDifferentialEvolutionSolver:
         with suppress_warnings() as sup:
             sup.filter(UserWarning)
             res = differential_evolution(
-                f, bounds, strategy='best1bin', seed=1234,
+                f, bounds, strategy='best1bin', rng=1234,
                 constraints=constraints, popsize=3, tol=0.05
             )
 
@@ -1260,7 +1264,7 @@ class TestDifferentialEvolutionSolver:
         assert_(np.all(res.x >= np.array(bounds)[:, 0]))
         assert_(np.all(res.x <= np.array(bounds)[:, 1]))
 
-    @pytest.mark.fail_slow(5)
+    @pytest.mark.fail_slow(10)
     def test_L5(self):
         # Lampinen ([5]) test problem 5
 
@@ -1279,7 +1283,7 @@ class TestDifferentialEvolutionSolver:
         bounds = [(0, 10)]*2
         constraints = (N)
 
-        res = differential_evolution(f, bounds, strategy='rand1bin', seed=1234,
+        res = differential_evolution(f, bounds, strategy='rand1bin', rng=1234,
                                      constraints=constraints)
 
         x_opt = (1.22797135, 4.24537337)
@@ -1291,7 +1295,7 @@ class TestDifferentialEvolutionSolver:
         assert_(np.all(res.x >= np.array(bounds)[:, 0]))
         assert_(np.all(res.x <= np.array(bounds)[:, 1]))
 
-    @pytest.mark.fail_slow(5)
+    @pytest.mark.fail_slow(10)
     def test_L6(self):
         # Lampinen ([5]) test problem 6
         def f(x):
@@ -1307,7 +1311,7 @@ class TestDifferentialEvolutionSolver:
         N = NonlinearConstraint(c1, 0, np.inf)
         bounds = [(13, 100), (0, 100)]
         constraints = (N)
-        res = differential_evolution(f, bounds, strategy='rand1bin', seed=1234,
+        res = differential_evolution(f, bounds, strategy='rand1bin', rng=1234,
                                      constraints=constraints, tol=1e-7)
         x_opt = (14.095, 0.84296)
         f_opt = -6961.814744
@@ -1346,7 +1350,7 @@ class TestDifferentialEvolutionSolver:
         bounds = [(78, 102), (33, 45)] + [(27, 45)]*3
         constraints = (N)
 
-        res = differential_evolution(f, bounds, strategy='rand1bin', seed=1234,
+        res = differential_evolution(f, bounds, strategy='rand1bin', rng=1234,
                                      constraints=constraints)
 
         # using our best solution, rather than Lampinen/Koziel. Koziel solution
@@ -1403,7 +1407,7 @@ class TestDifferentialEvolutionSolver:
             # huge amount of CPU time. Changing strategy to best1bin speeds
             # things up a lot
             res = differential_evolution(f, bounds, strategy='best1bin',
-                                         seed=1234, constraints=constraints,
+                                         rng=1234, constraints=constraints,
                                          maxiter=5000)
 
         x_opt = (679.9453, 1026.067, 0.1188764, -0.3962336)
@@ -1436,7 +1440,7 @@ class TestDifferentialEvolutionSolver:
 
         bounds = [(-1, 1)]*2
         constraints = (N)
-        res = differential_evolution(f, bounds, strategy='rand1bin', seed=1234,
+        res = differential_evolution(f, bounds, strategy='rand1bin', rng=1234,
                                      constraints=constraints)
 
         x_opt = [np.sqrt(2)/2, 0.5]
@@ -1451,7 +1455,7 @@ class TestDifferentialEvolutionSolver:
         assert_(np.all(res.x >= np.array(bounds)[:, 0]))
         assert_(np.all(res.x <= np.array(bounds)[:, 1]))
 
-    @pytest.mark.fail_slow(5)
+    @pytest.mark.fail_slow(10)
     def test_integrality(self):
         # test fitting discrete distribution to data
         rng = np.random.default_rng(6519843218105)
@@ -1472,7 +1476,7 @@ class TestDifferentialEvolutionSolver:
 
         res = differential_evolution(func, bounds, args=(dist, x),
                                      integrality=integrality, polish=False,
-                                     seed=rng)
+                                     rng=rng)
         # tolerance has to be fairly relaxed for the second parameter
         # because we're fitting a distribution to random variates.
         assert res.x[0] == 5
@@ -1481,7 +1485,7 @@ class TestDifferentialEvolutionSolver:
         # check that we can still use integrality constraints with polishing
         res2 = differential_evolution(func, bounds, args=(dist, x),
                                       integrality=integrality, polish=True,
-                                      seed=rng)
+                                      rng=rng)
 
         def func2(p, *args):
             n, dist, x = args
@@ -1541,7 +1545,8 @@ class TestDifferentialEvolutionSolver:
             DifferentialEvolutionSolver(f, bounds=bounds, polish=False,
                                         integrality=integrality)
 
-    @pytest.mark.fail_slow(5)
+    @pytest.mark.thread_unsafe
+    @pytest.mark.fail_slow(10)
     def test_vectorized(self):
         def quadratic(x):
             return np.sum(x**2)
@@ -1573,9 +1578,9 @@ class TestDifferentialEvolutionSolver:
 
         bounds = [(0, 10), (0, 10)]
         res1 = differential_evolution(rosen, bounds, updating='deferred',
-                                      seed=1)
+                                      rng=1)
         res2 = differential_evolution(rosen_vec, bounds, vectorized=True,
-                                      updating='deferred', seed=1)
+                                      updating='deferred', rng=1)
 
         # the two minimisation runs should be functionally equivalent
         assert_allclose(res1.x, res2.x)
@@ -1601,10 +1606,10 @@ class TestDifferentialEvolutionSolver:
         bounds = [(0, 10), (0, 10)]
 
         res1 = differential_evolution(rosen, bounds, updating='deferred',
-                                      seed=1, constraints=[nlc1, nlc2],
+                                      rng=1, constraints=[nlc1, nlc2],
                                       polish=False)
         res2 = differential_evolution(rosen_vec, bounds, vectorized=True,
-                                      updating='deferred', seed=1,
+                                      updating='deferred', rng=1,
                                       constraints=[nlc1, nlc2],
                                       polish=False)
         # the two minimisation runs should be functionally equivalent
@@ -1624,16 +1629,16 @@ class TestDifferentialEvolutionSolver:
                                         constraints=[c0, c1],
                                         maxiter=10,
                                         polish=False,
-                                        seed=864197532)
+                                        rng=864197532)
         assert result.success is False
         # The numerical value in the error message might be sensitive to
         # changes in the implementation.  It can be updated if the code is
         # changed.  The essential part of the test is that there is a number
         # after the '=', so if necessary, the text could be reduced to, say,
         # "MAXCV = 0.".
-        assert "MAXCV = 0.4" in result.message
+        assert "MAXCV = 0." in result.message
 
-    @pytest.mark.fail_slow(10)  # fail-slow exception by request - see gh-20806
+    @pytest.mark.fail_slow(20)  # fail-slow exception by request - see gh-20806
     def test_strategy_fn(self):
         # examines ability to customize strategy by mimicking one of the
         # in-built strategies
@@ -1674,7 +1679,7 @@ class TestDifferentialEvolutionSolver:
             mutation=mutation,
             maxiter=2,
             strategy=custom_strategy_fn,
-            seed=10,
+            rng=10,
             polish=False
         )
         assert solver.strategy is custom_strategy_fn
@@ -1696,4 +1701,3 @@ class TestDifferentialEvolutionSolver:
                 bounds,
                 strategy=custom_strategy_fn
             )
-
