@@ -6,11 +6,6 @@
 #include "sf_error.h"
 
 
-// Override some default BOOST policies.
-// These are required to ensure that the Boost function ibeta_inv
-// handles extremely small p values with precision comparable to the
-// Cephes incbi function.
-
 #include "boost/math/special_functions/beta.hpp"
 #include "boost/math/special_functions/erf.hpp"
 #include "boost/math/special_functions/powm1.hpp"
@@ -78,13 +73,46 @@ Real ibeta_wrap(Real a, Real b, Real x)
 {
     Real y;
 
-    if (isnan(a) || isnan(b) || isnan(x)) {
+    if (std::isnan(a) || std::isnan(b) || std::isnan(x)) {
         return NAN;
     }
-    if ((a <= 0) || (b <= 0) || (x < 0) || (x > 1)) {
+
+    if ((a < 0) || (b < 0) || (x < 0) || (x > 1)) {
         sf_error("betainc", SF_ERROR_DOMAIN, NULL);
         return NAN;
     }
+
+    /* In limiting cases, SciPy treats `betainc` as a two parameter family
+     * of functions of a single variable `x`, rather than as a function of
+     * three variables `a`, `b`, `x`. The limit ``(a, b) -> (a0, b0)`` of
+     * ``betainc(a, b, x)`` is treated as the pointwise limit in `x`.
+     */
+
+    if (((a == 0) && (b == 0)) || (std::isinf(a) && std::isinf(b))) {
+	/* In the limit (a, b) -> (0+, 0+), the Beta distribution converges
+	 * to a Bernoulli(p) distribution, where p depends on the path in
+	 * which (a, b) approaches (0+, 0+).
+	 * e.g. if a = t*b then the limiting distribution will be
+	 * Bernoulli(t / (t + 1)). The a = 0, b = 0 case is thus indeterminate.
+	 * A similar statement can be made for the limit (a, b) -> (inf, inf).
+	 */
+        return NAN;
+    }
+
+    if ((a == 0) || std::isinf(b)) {
+	/* Distribution in the limit a -> 0+, b > 0 is a point distribution
+	 * at x = 0. The same is true in the limit b -> inf for fixed a.
+	 */
+	return x > 0 ? 1 : 0;
+    }
+
+    if ((b == 0) || std::isinf(a)) {
+	/* Distribution in the limit b -> 0+, a > 0 is a point distribution
+	 * at x = 1. The same is true in the limit a -> inf for fixed b.
+	 */
+	return x < 1 ? 0 : 1;
+    }
+
     try {
         y = boost::math::ibeta(a, b, x, SpecialPolicy());
     } catch (const std::domain_error& e) {
@@ -122,13 +150,46 @@ Real ibetac_wrap(Real a, Real b, Real x)
 {
     Real y;
 
-    if (isnan(a) || isnan(b) || isnan(x)) {
+    if (std::isnan(a) || std::isnan(b) || std::isnan(x)) {
         return NAN;
     }
-    if ((a <= 0) || (b <= 0) || (x < 0) || (x > 1)) {
+
+    if ((a < 0) || (b < 0) || (x < 0) || (x > 1)) {
         sf_error("betaincc", SF_ERROR_DOMAIN, NULL);
         return NAN;
     }
+
+    /* In limiting cases, SciPy treats `betaincc` as a two parameter family
+     * of functions of a single variable `x`, rather than as a function of
+     * three variables `a`, `b`, `x`. The limit ``(a, b) -> (a0, b0)`` of
+     * ``betaincc(a, b, x)`` is treated as the pointwise limit in `x`.
+     */
+
+    if (((a == 0) && (b == 0)) || (std::isinf(a) && std::isinf(b))) {
+	/* In the limit (a, b) -> (0+, 0+), the Beta distribution converges
+	 * to a Bernoulli(p) distribution, where p depends on the path in
+	 * which (a, b) approaches (0+, 0+).
+	 * e.g. if a = t*b then the limiting distribution will be
+	 * Bernoulli(t / (t + 1)). The a = 0, b = 0 case is thus indeterminate.
+	 * A similar statement can be made for the limit (a, b) -> (inf, inf).
+	 */
+        return NAN;
+    }
+
+    if ((a == 0) || std::isinf(b)) {
+	/* Distribution in the limit a -> 0+, b > 0 is a point distribution
+	 * at x = 0. The same is true in the limit b -> inf for fixed a.
+	 */
+	return x > 0 ? 0 : 1;
+    }
+
+    if ((b == 0) || std::isinf(a)) {
+	/* Distribution in the limit b -> 0+, a > 0 is a point distribution
+	 * at x = 1.  The same is true in the limit a -> inf for fixed b.
+	 */
+	return x < 1 ? 1 : 0;
+    }
+
     try {
         y = boost::math::ibetac(a, b, x);
     } catch (const std::domain_error& e) {
@@ -166,7 +227,7 @@ Real ibeta_inv_wrap(Real a, Real b, Real p, const Policy& policy_)
 {
     Real y;
 
-    if (isnan(a) || isnan(b) || isnan(p)) {
+    if (std::isnan(a) || std::isnan(b) || std::isnan(p)) {
         return NAN;
     }
     if ((a <= 0) || (b <= 0) || (p < 0) || (p > 1)) {
@@ -210,7 +271,7 @@ Real ibetac_inv_wrap(Real a, Real b, Real p)
 {
     Real y;
 
-    if (isnan(a) || isnan(b) || isnan(p)) {
+    if (std::isnan(a) || std::isnan(b) || std::isnan(p)) {
         return NAN;
     }
     if ((a <= 0) || (b <= 0) || (p < 0) || (p > 1)) {
@@ -428,7 +489,7 @@ Real hyp1f1_wrap(Real a, Real b, Real x)
 {
     Real y;
 
-    if (isnan(a) || isnan(b) || isnan(x)) {
+    if (std::isnan(a) || std::isnan(b) || std::isnan(x)) {
         return NAN;
     }
     if (b <= 0 && std::trunc(b) == b) {
@@ -536,7 +597,7 @@ Real beta_ppf_wrap(const Real x, const Real a, const Real b)
         boost::math::policies::domain_error<boost::math::policies::ignore_error >,
         boost::math::policies::overflow_error<boost::math::policies::user_error >,
         boost::math::policies::evaluation_error<boost::math::policies::user_error >,
-        boost::math::policies::promote_float<false > > BetaPolicyForStats;
+        boost::math::policies::promote_double<false > > BetaPolicyForStats;
 
     return ibeta_inv_wrap(a, b, x, BetaPolicyForStats());
 }
@@ -593,13 +654,67 @@ invgauss_isf_double(double x, double mu, double s)
     return invgauss_isf_wrap(x, mu, s);
 }
 
+template<typename Real>
+Real
+cauchy_ppf_wrap(const Real p, const Real loc, const Real scale)
+{
+    return boost::math::quantile(
+        boost::math::cauchy_distribution<Real, StatsPolicy>(loc, scale), p);
+}
+
+float
+cauchy_ppf_float(float p, float loc, float scale)
+{
+    return cauchy_ppf_wrap(p, loc, scale);
+}
+
+double
+cauchy_ppf_double(double p, double loc, double scale)
+{
+    return cauchy_ppf_wrap(p, loc, scale);
+}
+
+template<typename Real>
+Real
+cauchy_isf_wrap(const Real p, const Real loc, const Real scale)
+{
+    return boost::math::quantile(boost::math::complement(
+        boost::math::cauchy_distribution<Real, StatsPolicy>(loc, scale), p));
+}
+
+float
+cauchy_isf_float(float p, float loc, float scale)
+{
+    return cauchy_isf_wrap(p, loc, scale);
+}
+
+double
+cauchy_isf_double(double p, double loc, double scale)
+{
+    return cauchy_isf_wrap(p, loc, scale);
+}
+
 template<typename Real>
 Real
 ncx2_pdf_wrap(const Real x, const Real k, const Real l)
 {
     if (std::isfinite(x)) {
+        try {
         return boost::math::pdf(
-            boost::math::non_central_chi_squared_distribution<Real, StatsPolicy>(k, l), x);
+            boost::math::non_central_chi_squared_distribution<Real, SpecialPolicy>(k, l), x);
+    }
+    catch (const std::overflow_error& e) {
+        sf_error("_ncx2_pdf", SF_ERROR_OVERFLOW, NULL);
+        return INFINITY;
+    }
+    catch (const std::underflow_error& e) {
+        sf_error("_ncx2_pdf", SF_ERROR_UNDERFLOW, NULL);
+        return 0;
+    }
+    catch (...) {
+        sf_error("_ncx2_pdf", SF_ERROR_OTHER, NULL);
+        return NAN;
+    }
     }
     return NAN; // inf or -inf returns NAN
 }
@@ -725,46 +840,93 @@ ncf_pdf_double(double x, double v1, double v2, double l)
 
 template<typename Real>
 Real
-ncf_cdf_wrap(const Real x, const Real v1, const Real v2, const Real l)
+ncf_cdf_wrap(const Real v1, const Real v2, const Real l, const Real x)
 {
-    if (std::isfinite(x)) {
-        return boost::math::cdf(
-            boost::math::non_central_f_distribution<Real, StatsPolicy>(v1, v2, l), x);
+    if (std::isnan(x) || std::isnan(v1) || std::isnan(v2) || std::isnan(l)) {
+	return NAN;
     }
-    // -inf => 0, inf => 1
-    return 1.0 - std::signbit(x);
+    if ((v1 <= 0) || (v2 <= 0) || (l < 0) || (x < 0)) {
+	sf_error("ncfdtr", SF_ERROR_DOMAIN, NULL);
+	return NAN;
+    }
+    if (std::isinf(x)) {
+	// inf => 1. We've already returned if x < 0, so this can only be +inf.
+	return 1.0;
+    }
+    Real y;
+    try {
+	y = boost::math::cdf(
+                boost::math::non_central_f_distribution<Real, SpecialPolicy>(v1, v2, l), x);
+    } catch (...) {
+	/* Boost was unable to produce a result. Can happen when one or both
+	 * of v1 and v2 is very small and x is very large. e.g.
+	 * ncdtr(1e-100, 3, 1.5, 1e100). */
+        sf_error("ncfdtr", SF_ERROR_NO_RESULT, NULL);
+        y = NAN;
+    }
+    if ((y < 0) || (y > 1)) {
+	/* Boost can return results far out of bounds when dfd and dfn are both large
+	 * and of similar magnitude. Return NAN if the result is out of bounds because
+	 * the answer cannot be trusted. */
+	sf_error("ncfdtr", SF_ERROR_NO_RESULT, NULL);
+        y = NAN;
+    }
+    return y;
 }
 
 float
-ncf_cdf_float(float x, float v1, float v2, float l)
+ncf_cdf_float(float v1, float v2, float l, float x)
 {
-    return ncf_cdf_wrap(x, v1, v2, l);
+    return ncf_cdf_wrap(v1, v2, l, x);
 }
 
 double
-ncf_cdf_double(double x, double v1, double v2, double l)
+ncf_cdf_double(double v1, double v2, double l, double x)
 {
-    return ncf_cdf_wrap(x, v1, v2, l);
+    return ncf_cdf_wrap(v1, v2, l, x);
 }
 
 template<typename Real>
 Real
-ncf_ppf_wrap(const Real x, const Real v1, const Real v2, const Real l)
+ncf_ppf_wrap(const Real v1, const Real v2, const Real l, const Real x)
 {
-    return boost::math::quantile<Real, StatsPolicy>(
-        boost::math::non_central_f_distribution<Real, StatsPolicy>(v1, v2, l), x);
+    if (std::isnan(x) || std::isnan(v1) || std::isnan(v2) || std::isnan(l)) {
+	return NAN;
+    }
+    if ((v1 <= 0) || (v2 <= 0) || (l < 0) || (x < 0) || (x > 1)) {
+	sf_error("ncfdtri", SF_ERROR_DOMAIN, NULL);
+	return NAN;
+    }
+    Real y;
+    try {
+	y = boost::math::quantile<Real, SpecialPolicy>(
+                boost::math::non_central_f_distribution<Real, SpecialPolicy>(v1, v2, l), x);
+    } catch (const std::domain_error& e) {
+        sf_error("ncfdtri", SF_ERROR_DOMAIN, NULL);
+        y = NAN;
+    } catch (const std::overflow_error& e) {
+        sf_error("ncfdtri", SF_ERROR_OVERFLOW, NULL);
+        y = INFINITY;
+    } catch (const std::underflow_error& e) {
+        sf_error("ncfdtri", SF_ERROR_UNDERFLOW, NULL);
+        y = 0;
+    } catch (...) {
+        sf_error("ncfdtri", SF_ERROR_NO_RESULT, NULL);
+        y = NAN;
+    }
+    return y;
 }
 
 float
-ncf_ppf_float(float x, float v1, float v2, float l)
+ncf_ppf_float(float v1, float v2, float l, float x)
 {
-    return ncf_ppf_wrap(x, v1, v2, l);
+    return ncf_ppf_wrap(v1, v2, l, x);
 }
 
 double
-ncf_ppf_double(double x, double v1, double v2, double l)
+ncf_ppf_double(double v1, double v2, double l, double x)
 {
-    return ncf_ppf_wrap(x, v1, v2, l);
+    return ncf_ppf_wrap(v1, v2, l, x);
 }
 
 template<typename Real>
@@ -869,46 +1031,114 @@ ncf_kurtosis_excess_double(double v1, double v2, double l)
 
 template<typename Real>
 Real
-nct_cdf_wrap(const Real x, const Real v, const Real l)
+nct_cdf_wrap(const Real v, const Real l, const Real x)
 {
-    if (std::isfinite(x)) {
-        return boost::math::cdf(
-            boost::math::non_central_t_distribution<Real, StatsPolicy>(v, l), x);
+    if (std::isnan(x) || std::isnan(v) || std::isnan(l)) {
+	return NAN;
     }
-    // -inf => 0, inf => 1
-    return 1.0 - std::signbit(x);
+    if (v <= 0) {
+	sf_error("nctdtr", SF_ERROR_DOMAIN, NULL);
+	return NAN;
+    }
+    if (std::isinf(x)) {
+	return  (x > 0) ? 1.0 : 0.0;
+    }
+    Real y;
+    try {
+	y = boost::math::cdf(
+                boost::math::non_central_t_distribution<Real, SpecialPolicy>(v, l), x);
+    } catch (...) {
+	/* Boost was unable to produce a result. */
+        sf_error("nctdtr", SF_ERROR_NO_RESULT, NULL);
+        y = NAN;
+    }
+    if ((y < 0) || (y > 1)) {
+	/* Result must be between 0 and 1 to be a valid CDF value.
+       Return NAN if the result is out of bounds because the answer cannot be trusted. */
+	    sf_error("nctdtr", SF_ERROR_NO_RESULT, NULL);
+        y = NAN;
+    }
+    return y;
 }
 
 float
-nct_cdf_float(float x, float v, float l)
+nct_cdf_float(float v, float l, float x)
 {
-    return nct_cdf_wrap(x, v, l);
+    return nct_cdf_wrap(v, l, x);
 }
 
 double
-nct_cdf_double(double x, double v, double l)
+nct_cdf_double(double v, double l, double x)
 {
-    return nct_cdf_wrap(x, v, l);
+    return nct_cdf_wrap(v, l, x);
 }
 
 template<typename Real>
 Real
-nct_ppf_wrap(const Real x, const Real v, const Real l)
+nct_pdf_wrap(const Real x, const Real v, const Real l)
 {
-    return boost::math::quantile(
-        boost::math::non_central_t_distribution<Real, StatsPolicy>(v, l), x);
+    if (std::isfinite(x)) {
+        return boost::math::pdf(
+            boost::math::non_central_t_distribution<Real, StatsPolicy>(v, l), x);
+    }
+    return NAN;
 }
 
 float
-nct_ppf_float(float x, float v, float l)
+nct_pdf_float(float x, float v, float l)
 {
-    return nct_ppf_wrap(x, v, l);
+    return nct_pdf_wrap(x, v, l);
 }
 
 double
-nct_ppf_double(double x, double v, double l)
+nct_pdf_double(double x, double v, double l)
 {
-    return nct_ppf_wrap(x, v, l);
+    return nct_pdf_wrap(x, v, l);
+}
+
+template<typename Real>
+Real
+nct_ppf_wrap(const Real v, const Real l, const Real x)
+{
+    if (std::isnan(x) || std::isnan(v) || std::isnan(l)) {
+        return NAN;
+    }
+    if (v <= 0 || x < 0 || x > 1) {
+        sf_error("nctdtrit", SF_ERROR_DOMAIN, NULL);
+        return NAN;
+    }
+    Real y;
+    try {
+	y = boost::math::quantile(
+        boost::math::non_central_t_distribution<Real, SpecialPolicy>(v, l), x);
+    }
+    catch (const std::domain_error& e) {
+        sf_error("nctdtrit", SF_ERROR_DOMAIN, NULL);
+        y = NAN;
+    } catch (const std::overflow_error& e) {
+        sf_error("nctdtrit", SF_ERROR_OVERFLOW, NULL);
+        y = INFINITY;
+    } catch (const std::underflow_error& e) {
+        sf_error("nctdtrit", SF_ERROR_UNDERFLOW, NULL);
+        y = 0; 
+    } catch (...) {
+	/* Boost was unable to produce a result. */
+        sf_error("nctdtrit", SF_ERROR_NO_RESULT, NULL);
+        y = NAN;
+    }
+    return y;
+}
+
+float
+nct_ppf_float(float v, float l, float x)
+{
+    return nct_ppf_wrap(v, l, x);
+}
+
+double
+nct_ppf_double(double v, double l, double x)
+{
+    return nct_ppf_wrap(v, l, x);
 }
 
 template<typename Real>
@@ -1102,6 +1332,9 @@ binom_cdf_wrap(const Real x, const Real n, const Real p)
         return boost::math::cdf(
             boost::math::binomial_distribution<Real, StatsPolicy>(n, p), x);
     }
+    if (std::isnan(x)) {
+	return std::numeric_limits<double>::quiet_NaN();
+    }
     // -inf => 0, inf => 1
     return 1 - std::signbit(x);
 }
@@ -1436,4 +1669,119 @@ hypergeom_skewness_double(double n, double N, double M)
     return boost::math::skewness(boost::math::hypergeometric_distribution<double, StatsPolicy>(n, N, M));
 }
 
+template<typename Real>
+Real
+landau_pdf_wrap(const Real x, const Real loc, const Real scale)
+{
+    if (std::isfinite(x)) {
+        return boost::math::pdf(
+            boost::math::landau_distribution<Real, StatsPolicy>(loc, scale), x);
+    }
+    return NAN;
+}
+
+float
+landau_pdf_float(float x, float loc, float scale)
+{
+    return landau_pdf_wrap(x, loc, scale);
+}
+
+double
+landau_pdf_double(double x, double loc, double scale)
+{
+    return landau_pdf_wrap(x, loc, scale);
+}
+
+template<typename Real>
+Real
+landau_cdf_wrap(const Real x, const Real loc, const Real scale)
+{
+    if (std::isfinite(x)) {
+        return boost::math::cdf(
+            boost::math::landau_distribution<Real, StatsPolicy>(loc, scale), x);
+    }
+    return NAN;
+}
+
+float
+landau_cdf_float(float x, float loc, float scale)
+{
+    return landau_cdf_wrap(x, loc, scale);
+}
+
+double
+landau_cdf_double(double x, double loc, double scale)
+{
+    return landau_cdf_wrap(x, loc, scale);
+}
+
+template<typename Real>
+Real
+landau_sf_wrap(const Real x, const Real loc, const Real scale)
+{
+    if (std::isfinite(x)) {
+        return boost::math::cdf(boost::math::complement(
+            boost::math::landau_distribution<Real, StatsPolicy>(loc, scale), x));
+    }
+    return NAN;
+}
+
+float
+landau_sf_float(float x, float loc, float scale)
+{
+    return landau_sf_wrap(x, loc, scale);
+}
+
+double
+landau_sf_double(double x, double loc, double scale)
+{
+    return landau_sf_wrap(x, loc, scale);
+}
+
+template<typename Real>
+Real
+landau_ppf_wrap(const Real p, const Real loc, const Real scale)
+{
+    if (std::isfinite(p)) {
+        return boost::math::quantile(
+            boost::math::landau_distribution<Real, StatsPolicy>(loc, scale), p);
+    }
+    return NAN;
+}
+
+float
+landau_ppf_float(float p, float loc, float scale)
+{
+    return landau_ppf_wrap(p, loc, scale);
+}
+
+double
+landau_ppf_double(double p, double loc, double scale)
+{
+    return landau_ppf_wrap(p, loc, scale);
+}
+
+template<typename Real>
+Real
+landau_isf_wrap(const Real p, const Real loc, const Real scale)
+{
+    if (std::isfinite(p)) {
+        return boost::math::quantile(boost::math::complement(
+            boost::math::landau_distribution<Real, StatsPolicy>(loc, scale), p));
+    }
+    return NAN;
+}
+
+float
+landau_isf_float(float p, float loc, float scale)
+{
+    return landau_isf_wrap(p, loc, scale);
+}
+
+double
+landau_isf_double(double p, double loc, double scale)
+{
+    return landau_isf_wrap(p, loc, scale);
+}
+
 #endif
